@@ -22,12 +22,12 @@ wants to deal with as an array.
         name => $name,
         tag  => $tag,
         base => $base,
-        alt  => $alt,
+        attr => $attr_href,
     } );
 
 For compatibility, this older interface is also supported:
 
- new( $url, $text, $name, $tag, $base, $alt )
+ new( $url, $text, $name, $tag, $base, $attr_href )
 
 Creates and returns a new C<WWW::Mechanize::Link> object.
 
@@ -38,7 +38,7 @@ sub new {
 
     # new style API
     my %p;
-    my ($url,$text,$name,$tag,$base,$alt);
+    my ($url,$text,$name,$tag,$base,$attrs);
 
     if ( ref $_[0] eq 'HASH' ) {
         %p =  %{ $_[0] }; 
@@ -47,15 +47,15 @@ sub new {
         $name = $p{name};
         $tag  = $p{tag};
         $base = $p{base};
-        $alt  = $p{alt};
+        $attrs = $p{attrs};
     }
     else {
-        ($url,$text,$name,$tag,$base,$alt) = @_; 
+        ($url,$text,$name,$tag,$base,$attrs) = @_; 
     }
 
     # The order of the first four must stay as they are for
     # compatibility with older code.
-    my $self = [$url,$text,$name,$tag,$base,$alt];
+    my $self = [$url,$text,$name,$tag,$base,$attrs];
 
     bless $self, $class;
 
@@ -84,18 +84,18 @@ Tag name (either "a", "frame" or "iframe").
 
 Base URL to which the links are relative.
 
-=head2 $link->alt()
+=head2 $link->attrs()
 
-Alt attribute contents. Useful for 'area' tags.
+Returns hash ref of all the attributes and attribute values in the tag. 
 
 =cut
 
-sub url  { return ($_[0])->[0]; }
-sub text { return ($_[0])->[1]; }
-sub name { return ($_[0])->[2]; }
-sub tag  { return ($_[0])->[3]; }
-sub base { return ($_[0])->[4]; }
-sub alt  { return ($_[0])->[5]; }
+sub url   { return ($_[0])->[0]; }
+sub text  { return ($_[0])->[1]; }
+sub name  { return ($_[0])->[2]; }
+sub tag   { return ($_[0])->[3]; }
+sub base  { return ($_[0])->[4]; }
+sub attrs { return ($_[0])->[5]; }
 
 =head2 $link->URI()
 
